@@ -2708,7 +2708,7 @@ store_print_settings (GeditTab      *tab,
 static void
 done_printing_cb (GeditPrintJob       *job,
 		  GeditPrintJobResult  result,
-		  const GError        *error,
+		  GError              *error,
 		  GeditTab            *tab)
 {
 	GeditView *view;
@@ -2717,13 +2717,19 @@ done_printing_cb (GeditPrintJob       *job,
 			  tab->priv->state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW ||
 			  tab->priv->state == GEDIT_TAB_STATE_PRINTING);
 
-	/* TODO: check status and error */
-
 	if (result == GEDIT_PRINT_JOB_RESULT_OK)
 	{
 		store_print_settings (tab, job);
 	}
 
+	/* TODO Show the error in an info bar. */
+	if (error != NULL)
+	{
+		g_warning ("Printing error: %s", error->message);
+		g_error_free (error);
+		error = NULL;
+	}
+
 	close_printing (tab);
 
 	view = gedit_tab_get_view (tab);
@@ -2769,7 +2775,7 @@ print_cancelled (GtkWidget *bar,
 }
 
 static void
-show_printing_info_bar (GeditTab *tab)
+add_printing_info_bar (GeditTab *tab)
 {
 	GtkWidget *bar;
 
@@ -2783,6 +2789,9 @@ show_printing_info_bar (GeditTab *tab)
 			  tab);
 
 	set_info_bar (tab, bar, GTK_RESPONSE_NONE);
+
+	/* hide until we start printing */
+	gtk_widget_hide (bar);
 }
 
 void
@@ -2812,10 +2821,7 @@ _gedit_tab_print (GeditTab *tab)
 
 	tab->priv->print_job = gedit_print_job_new (view);
 
-	show_printing_info_bar (tab);
-
-	/* hide until we start printing */
-	gtk_widget_hide (tab->priv->info_bar);
+	add_printing_info_bar (tab);
 
 	g_signal_connect_object (tab->priv->print_job,
 				 "printing",
