@@ -42,6 +42,7 @@
 #include "gedit-automatic-spell-checker.h"
 
 #ifdef G_OS_WIN32
+#include <gedit/gedit-metadata-manager.h>
 #define GEDIT_METADATA_ATTRIBUTE_SPELL_LANGUAGE "spell-language"
 #define GEDIT_METADATA_ATTRIBUTE_SPELL_ENABLED  "spell-enabled"
 #else
@@ -89,16 +90,15 @@ G_DEFINE_DYNAMIC_TYPE_EXTENDED (GeditSpellPlugin,
 							       gedit_window_activatable_iface_init)
 				G_ADD_PRIVATE_DYNAMIC (GeditSpellPlugin))
 
-static void	spell_cb			(GSimpleAction *action, GVariant *parameter, gpointer data);
-static void	set_language_cb			(GSimpleAction *action, GVariant *parameter, gpointer data);
-static void	auto_spell_activate_cb		(GSimpleAction *action, GVariant *parameter, gpointer data);
-static void	auto_spell_change_state_cb	(GSimpleAction *action, GVariant *state, gpointer data);
+static void	spell_cb	(GSimpleAction *action, GVariant *parameter, gpointer data);
+static void	set_language_cb	(GSimpleAction *action, GVariant *parameter, gpointer data);
+static void	auto_spell_cb	(GSimpleAction *action, GVariant *state, gpointer data);
 
 static GActionEntry action_entries[] =
 {
 	{ "check-spell", spell_cb },
 	{ "config-spell", set_language_cb },
-	{ "auto-spell", auto_spell_activate_cb, NULL, "false", auto_spell_change_state_cb }
+	{ "auto-spell", NULL, NULL, "false", auto_spell_cb }
 };
 
 static GQuark spell_checker_id = 0;
@@ -467,7 +467,9 @@ goto_next_word (GeditDocument *doc)
 }
 
 static gchar *
-get_next_misspelled_word (GeditView *view)
+get_next_misspelled_word (GeditView *view,
+                          gint      *word_start_offset,
+                          gint      *word_end_offset)
 {
 	GeditDocument *doc;
 	CheckRange *range;
@@ -512,42 +514,56 @@ get_next_misspelled_word (GeditView *view)
 
 	if (word != NULL)
 	{
-		GtkTextIter s, e;
-
 		range->mw_start = start;
 		range->mw_end = end;
-
-		gedit_debug_message (DEBUG_PLUGINS, "Select [%d, %d]", start, end);
-
-		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &s, start);
-		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &e, end);
-
-		gtk_text_buffer_select_range (GTK_TEXT_BUFFER (doc), &s, &e);
-
-		gedit_view_scroll_to_cursor (view);
+		*word_start_offset = start;
+		*word_end_offset = end;
 	}
 	else
 	{
 		range->mw_start = -1;
 		range->mw_end = -1;
+		*word_start_offset = -1;
+		*word_end_offset = -1;
 	}
 
 	return word;
 }
 
+static void
+select_misspelled_word (GeditView *view,
+                        gint       word_start_offset,
+                        gint       word_end_offset)
+{
+	GtkTextBuffer *buffer;
+	GtkTextIter word_start_iter;
+	GtkTextIter word_end_iter;
+
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+
+	gtk_text_buffer_get_iter_at_offset (buffer, &word_start_iter, word_start_offset);
+	gtk_text_buffer_get_iter_at_offset (buffer, &word_end_iter, word_end_offset);
+
+	gedit_debug_message (DEBUG_PLUGINS, "Select [%d, %d]", word_start_offset, word_end_offset);
+	gtk_text_buffer_select_range (buffer, &word_start_iter, &word_end_iter);
+
+	gedit_view_scroll_to_cursor (view);
+}
+
 static void
 ignore_cb (GeditSpellCheckerDialog *dlg,
 	   const gchar             *w,
 	   GeditView               *view)
 {
 	gchar *word = NULL;
+	gint word_start_offset, word_end_offset;
 
 	gedit_debug (DEBUG_PLUGINS);
 
 	g_return_if_fail (w != NULL);
 	g_return_if_fail (view != NULL);
 
-	word = get_next_misspelled_word (view);
+	word = get_next_misspelled_word (view, &word_start_offset, &word_end_offset);
 	if (word == NULL)
 	{
 		gedit_spell_checker_dialog_set_completed (dlg);
@@ -555,6 +571,8 @@ ignore_cb (GeditSpellCheckerDialog *dlg,
 		return;
 	}
 
+	select_misspelled_word (view, word_start_offset, word_end_offset);
+
 	gedit_spell_checker_dialog_set_misspelled_word (GEDIT_SPELL_CHECKER_DIALOG (dlg),
 							word,
 							-1);
@@ -765,6 +783,7 @@ spell_cb (GSimpleAction *action,
 	GeditSpellChecker *spell;
 	GtkWidget *dlg;
 	GtkTextIter start, end;
+	gint word_start_offset, word_end_offset;
 	gchar *word;
 	gchar *data_dir;
 
@@ -805,7 +824,7 @@ spell_cb (GSimpleAction *action,
 
 	set_check_range (doc, &start, &end);
 
-	word = get_next_misspelled_word (view);
+	word = get_next_misspelled_word (view, &word_start_offset, &word_end_offset);
 	if (word == NULL)
 	{
 		GtkWidget *statusbar;
@@ -840,6 +859,8 @@ spell_cb (GSimpleAction *action,
 	g_free (word);
 
 	gtk_widget_show (dlg);
+	select_misspelled_word (view, word_start_offset, word_end_offset);
+
 }
 
 static void
@@ -875,52 +896,9 @@ set_auto_spell (GeditWindow   *window,
 }
 
 static void
-auto_spell_activate_cb (GSimpleAction *action,
-			GVariant      *parameter,
-			gpointer       data)
-{
-	GeditSpellPlugin *plugin = GEDIT_SPELL_PLUGIN (data);
-	GeditSpellPluginPrivate *priv = plugin->priv;
-	GVariant *state;
-	gboolean active;
-	GeditView *view;
-
-	gedit_debug (DEBUG_PLUGINS);
-
-	state = g_action_get_state (G_ACTION (action));
-	g_return_if_fail (state != NULL);
-
-	active = g_variant_get_boolean (state);
-	g_variant_unref (state);
-
-	/* We must toggle ourself the value. */
-	active = !active;
-	g_action_change_state (G_ACTION (action), g_variant_new_boolean (active));
-
-	view = gedit_window_get_active_view (priv->window);
-	if (view != NULL)
-	{
-		GeditDocument *doc;
-
-		doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
-
-		/* Set metadata in the "activate" handler, not in "change-state"
-		 * because "change-state" is called every time the state
-		 * changes, not specifically when the user has changed the state
-		 * herself. For example "change-state" is called to initialize
-		 * the sate to the default value specified in the GActionEntry.
-		 */
-		gedit_document_set_metadata (doc,
-					     GEDIT_METADATA_ATTRIBUTE_SPELL_ENABLED,
-					     active ? "1" : NULL,
-					     NULL);
-	}
-}
-
-static void
-auto_spell_change_state_cb (GSimpleAction *action,
-			    GVariant      *state,
-			    gpointer       data)
+auto_spell_cb (GSimpleAction  *action,
+               GVariant       *state,
+               gpointer        data)
 {
 	GeditSpellPlugin *plugin = GEDIT_SPELL_PLUGIN (data);
 	GeditSpellPluginPrivate *priv = plugin->priv;
@@ -936,6 +914,14 @@ auto_spell_change_state_cb (GSimpleAction *action,
 	view = gedit_window_get_active_view (priv->window);
 	if (view != NULL)
 	{
+		GeditDocument *doc;
+
+		doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
+
+		gedit_document_set_metadata (doc,
+					     GEDIT_METADATA_ATTRIBUTE_SPELL_ENABLED,
+					     active ? "1" : NULL, NULL);
+
 		set_auto_spell (priv->window, view, active);
 		g_simple_action_set_state (action, g_variant_new_boolean (active));
 	}
